@@ -27,6 +27,11 @@
 #include "pygobject-private.h"
 
 static PyObject *gerror_exc = NULL;
+static const gchar *pyginterface_type_id   = "PyGInterface::type";
+GQuark pyginterface_type_key  = 0;
+
+static void pyg_flags_add_constants(PyObject *module, GType flags_type,
+				    const gchar *strip_prefix);
 
 /* -------------- GDK threading hooks ---------------------------- */
 
@@ -210,7 +215,9 @@ pyobject_free(gpointer boxed)
 {
     PyObject *object = boxed;
 
+    pyg_block_threads();
     Py_DECREF(object);
+    pyg_unblock_threads();
 }
 
 
@@ -229,7 +236,7 @@ pyg_interface_init(PyObject *self, PyObject *args, PyObject *kwargs)
     return -1;
 }
 
-static PyTypeObject PyGInterface_Type = {
+PyTypeObject PyGInterface_Type = {
     PyObject_HEAD_INIT(NULL)
     0,                                  /* ob_size */
     "gobject.GInterface",               /* tp_name */
@@ -270,7 +277,7 @@ static PyTypeObject PyGInterface_Type = {
     (initproc)pyg_interface_init,	/* tp_init */
     (allocfunc)0,			/* tp_alloc */
     (newfunc)0,				/* tp_new */
-    object_free,			/* tp_free */
+    (freefunc)object_free,		/* tp_free */
     (inquiry)0,				/* tp_is_gc */
     (PyObject *)0,			/* tp_bases */
 };
@@ -306,7 +313,10 @@ pyg_register_interface(PyObject *dict, const gchar *class_name,
         Py_DECREF(o);
     }
 
+    g_type_set_qdata(gtype, pyginterface_type_key, type);
+    
     PyDict_SetItemString(dict, (char *)class_name, (PyObject *)type);
+    
 }
 
 /* -------------- GMainContext objects ---------------------------- */
@@ -351,8 +361,10 @@ _wrap_g_main_context_iteration (PyGMainContext *self, PyObject *args)
 			  &may_block))
 	return NULL;
 	
+    pyg_unblock_threads();
     py_ret = g_main_context_iteration(self->context, may_block)
 	? Py_True : Py_False;
+    pyg_block_threads();
     
     Py_INCREF(py_ret);
     return py_ret;
@@ -496,7 +508,9 @@ _wrap_g_main_loop_quit (PyGMainLoop *self)
 static PyObject *
 _wrap_g_main_loop_run (PyGMainLoop *self)
 {
+    pyg_unblock_threads();
     g_main_loop_run(self->loop);
+    pyg_block_threads();
     
     Py_INCREF(Py_None);
     return Py_None;
@@ -1135,7 +1149,7 @@ pyg_type_register(PyObject *self, PyObject *args)
     PyTypeObject *class;
     GType parent_type, instance_type;
     gchar *type_name = NULL;
-    gint i;
+    gint i, name_serial;
     GTypeQuery query;
     GTypeInfo type_info = {
 	0,    /* class_size */
@@ -1165,22 +1179,34 @@ pyg_type_register(PyObject *self, PyObject *args)
 	return NULL;
     }
 
-    /* make name for new widget */
-    module = PyObject_GetAttrString((PyObject *)class, "__module__");
-    if (module && PyString_Check(module)) {
-	type_name = g_strconcat(PyString_AsString(module), ".",
-				class->tp_name, NULL);
-    } else {
-	if (module)
-	    Py_DECREF(module);
-	else
-	    PyErr_Clear();
-	type_name = g_strdup(class->tp_name);
+      /* make name for new GType */
+    name_serial = 1;
+    while (name_serial < 1000) /* give up after 1000 tries, just in case.. */
+    {
+        char name_serial_str[16];
+
+        snprintf(name_serial_str, 16, "-v%i", name_serial);
+        module = PyObject_GetAttrString((PyObject *)class, "__module__");
+        if (module && PyString_Check(module)) {
+            type_name = g_strconcat(PyString_AsString(module), ".",
+                                    class->tp_name,
+                                    name_serial > 1? name_serial_str : NULL,
+                                    NULL);
+        } else {
+            if (module)
+                Py_DECREF(module);
+            else
+                PyErr_Clear();
+            type_name = g_strdup(class->tp_name);
+        }
+          /* convert '.' in type name to '+', which isn't banned (grumble) */
+        for (i = 0; type_name[i] != '\0'; i++)
+            if (type_name[i] == '.')
+                type_name[i] = '+';
+        if (g_type_from_name(type_name) == 0)
+            break;              /* we now have a unique name */
+        ++name_serial;
     }
-    /* convert '.' in type name to '+', which isn't banned (grumble) */
-    for (i = 0; type_name[i] != '\0'; i++)
-	if (type_name[i] == '.')
-	    type_name[i] = '+';
 
     /* set class_data that will be passed to the class_init function. */
     type_info.class_data = class;
@@ -1785,9 +1811,11 @@ pyg_enum_add_constants(PyObject *module, GType enum_type,
     GEnumClass *eclass;
     guint i;
 
-    /* a more useful warning */
     if (!G_TYPE_IS_ENUM(enum_type)) {
-	g_warning("`%s' is not an enum type", g_type_name(enum_type));
+	if (G_TYPE_IS_FLAGS(enum_type))	/* See bug #136204 */
+	    pyg_flags_add_constants(module, enum_type, strip_prefix);
+	else
+	    g_warning("`%s' is not an enum type", g_type_name(enum_type));
 	return;
     }
     g_return_if_fail (strip_prefix != NULL);
@@ -1822,9 +1850,11 @@ pyg_flags_add_constants(PyObject *module, GType flags_type,
     GFlagsClass *fclass;
     guint i;
 
-    /* a more useful warning */
     if (!G_TYPE_IS_FLAGS(flags_type)) {
-	g_warning("`%s' is not an flags type", g_type_name(flags_type));
+	if (G_TYPE_IS_ENUM(flags_type))	/* See bug #136204 */
+	    pyg_enum_add_constants(module, flags_type, strip_prefix);
+	else
+	    g_warning("`%s' is not an flags type", g_type_name(flags_type));
 	return;
     }
     g_return_if_fail (strip_prefix != NULL);
@@ -1891,6 +1921,42 @@ pyg_error_check(GError **error)
     return FALSE;
 }
 
+
+static PyObject *
+_pyg_strv_from_gvalue(const GValue *value)
+{
+    gchar    **argv = (gchar **) g_value_get_boxed(value);
+    int        argc, i;
+    PyObject  *py_argv;
+
+    for (argc = -1; argv[++argc];);
+    py_argv = PyList_New(argc);
+    for (i = 0; i < argc; ++i)
+	PyList_SET_ITEM(py_argv, i, PyString_FromString(argv[i]));
+    return py_argv;
+}
+
+static int
+_pyg_strv_to_gvalue(GValue *value, PyObject *obj)
+{
+    int     argc, i;
+    gchar **argv;
+
+    if (!PySequence_Check(obj)) return -1;
+    argc = PySequence_Length(obj);
+    for (i = 0; i < argc; ++i)
+	if (!PyString_Check(PySequence_Fast_GET_ITEM(obj, i)))
+	    return -1;
+    argv = g_new(gchar *, argc + 1);
+    for (i = 0; i < argc; ++i)
+	argv[i] = g_strdup(PyString_AsString(PySequence_Fast_GET_ITEM(obj, i)));
+    argv[i] = NULL;
+    g_value_init(value, G_TYPE_STRV);
+    g_value_take_boxed(value, argv);
+    return 0;
+}
+
+
 /* ----------------- gobject module initialisation -------------- */
 
 struct _PyGObject_Functions pygobject_api_functions = {
@@ -1932,6 +1998,9 @@ struct _PyGObject_Functions pygobject_api_functions = {
   pyg_set_thread_block_funcs,
   (PyGThreadBlockFunc)0, /* block_threads */
   (PyGThreadBlockFunc)0, /* unblock_threads */
+
+  &PyGParamSpec_Type,
+  pyg_param_spec_new,
 };
 
 DL_EXPORT(void)
@@ -1979,6 +2048,7 @@ initgobject(void)
 			 pyg_object_descr_doc_get());
     PyDict_SetItemString(PyGInterface_Type.tp_dict, "__gdoc__",
 			 pyg_object_descr_doc_get());
+    pyginterface_type_key = g_quark_from_static_string(pyginterface_type_id);
 
     PyGBoxed_Type.ob_type = &PyType_Type;
     PyGBoxed_Type.tp_alloc = PyType_GenericAlloc;
@@ -2014,11 +2084,18 @@ initgobject(void)
 			 o=pyg_type_wrapper_new(G_TYPE_POINTER));
     Py_DECREF(o);
 
+    /* glib version */
     tuple = Py_BuildValue ("(iii)", glib_major_version, glib_minor_version,
 			   glib_micro_version);
     PyDict_SetItemString(d, "glib_version", tuple);    
     Py_DECREF(tuple);
 
+    /* pygtk version */
+    tuple = Py_BuildValue ("(iii)", PYGTK_MAJOR_VERSION, PYGTK_MINOR_VERSION,
+			   PYGTK_MICRO_VERSION);
+    PyDict_SetItemString(d, "pygtk_version", tuple);
+    Py_DECREF(tuple);
+
     /* for addon libraries ... */
     PyDict_SetItemString(d, "_PyGObject_API",
 			 o=PyCObject_FromVoidPtr(&pygobject_api_functions,NULL));
@@ -2075,4 +2152,8 @@ initgobject(void)
     PyModule_AddObject(m, "TYPE_PARAM", pyg_type_wrapper_new(G_TYPE_PARAM));
     PyModule_AddObject(m, "TYPE_OBJECT", pyg_type_wrapper_new(G_TYPE_OBJECT));
     PyModule_AddObject(m, "TYPE_PYOBJECT", pyg_type_wrapper_new(PY_TYPE_OBJECT));
+
+    pyg_register_boxed_custom(G_TYPE_STRV,
+			      _pyg_strv_from_gvalue,
+			      _pyg_strv_to_gvalue);
 }
